﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;

namespace MTGRenamer
{
    internal static class CardImage
    {
        public static Utils.LocationType ImageType = Utils.LocationType.Wagic;
        public static string ImagePath = string.Empty;

        public static Image Image(this Card card)
        {
            return GetImage(card);
        }

        private static Image GetImage(Card card)
        {
            switch (ImageType)
            {
                case Utils.LocationType.MagicAlbum:
                    return GetMagicAlbumImage(card);
                case Utils.LocationType.Wagic:
                    return GetWagicImage(card);
                case Utils.LocationType.MagicWorkstation:
                    return GetWokstationImage(card);
            }

            return null;
        }

        private static System.Drawing.Image GetWokstationImage(Card card)
        {
            string ImageZip = card.GetWorkstationPath(ImagePath);

            if (File.Exists(ImageZip))
                return System.Drawing.Image.FromFile(ImageZip);

            return null;
        }

        private static Image GetMagicAlbumImage(Card card)
        {
            string ImageZip = card.GetMagicAlbumPath(ImagePath);

            if (File.Exists(ImageZip))
                return System.Drawing.Image.FromFile(ImageZip);

            return null;
        }

        private static Image GetWagicImage(Card card)
        {
            //string ImageZip = Path.Combine(Utils.ImagePath, "sets");
            string ImageZip = Path.Combine(ImagePath, card.Set.WagicID);
            ImageZip = Path.Combine(ImageZip, card.Set.WagicID + ".zip");
            using (MemoryStream stream = new MemoryStream())
            {
                if (File.Exists(ImageZip))
                    using (ZipFile zip = new ZipFile(ImageZip))
                    {
                        ZipEntry e = (from i in zip.Entries
                                      where i.FileName.Contains(card.ID) && !i.FileName.Contains("thumbnail")
                                      select i).FirstOrDefault();
                        if (e != null)
                            e.Extract(stream);
                    }
                if (stream.Capacity > 0)
                    return System.Drawing.Image.FromStream(stream);
            }

            return null;
        }
    }
}
