﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.IO;
using System.Reflection;

namespace MTGRenamer
{
    public class Database
    {
        protected static string DBPath = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), "GathererDownloaderDB.xml");

        public static List<Token> TokensOnly { get; private set; }
        private static List<Card> _Cards;
        public static List<Card> Cards
        {
            get
            {
                if (_Cards == null)
                {
                    _Cards = new List<Card>();
                    LoadDatabase();
                }
                return _Cards;
            }
            private set
            {
                _Cards = value;
            }
        }

        public static void LoadDatabase()
        {
            LoadDatabase(DBPath);
        }

        public static void LoadDatabase(string path)
        {
            XElement doc = XElement.Load(path);
            LoadDatabase(doc);
        }

        private static void LoadDatabase(XElement doc)
        {
            IEnumerable<XElement> cards = doc.Element("cards").Elements("card");
            _Cards = new List<Card>();

            foreach (var item in cards)
            {
                string id = item.Element("id") == null ? string.Empty : item.Element("id").Value;
                string name = item.Element("name") == null ? string.Empty : item.Element("name").Value;
                string set = item.Element("set") == null ? string.Empty : item.Element("set").Value;
                string altart = item.Element("altart") == null ? string.Empty : item.Element("altart").Value;
                string cardnum = item.Element("cardnum") == null ? string.Empty : item.Element("cardnum").Value;
                string cardText = item.Element("rules") == null ? string.Empty : item.Element("rules").Value;
                string type = item.Element("type") == null ? string.Empty : item.Element("type").Value;
                string rarity = item.Element("rarity") == null ? string.Empty : item.Element("rarity").Value;
                Card newCard = new Card(name, set) { ID = id, AltArt = altart, CardNumber = cardnum, CardText = cardText, Type = type, Rarity = rarity };
                _Cards.Add(newCard);

            }
        }

        public static void LoadTokensOnly()
        {
            TokensOnly = new List<Token>();
            TokenGrabber grabber = new TokenGrabber();
            IEnumerable<Token> token = grabber.GrabList();
            TokensOnly.AddRange(token);
        }

        public static void AddTokensToCards(IEnumerable<Card> cards)
        {
            if (TokensOnly == null || TokensOnly.Count == 0)
                LoadTokensOnly();

            Cards.AddRange(cards);
        }

        public static List<Card> AddTokensReverse(IEnumerable<Card> cards)
        {
            List<Card> cardsToProcess = new List<Card>(cards);

            using (new HourGlass())
            {
                var CardsAffected = cardsToProcess.Where(x => x.NeededTokens.Count > 0).ToList();

                for (int i = 0; i < CardsAffected.Count(); i++)
                {
                    Card item = CardsAffected[i];

                    //Change the Set for each Token so it is copied in the right place
                    IEnumerable<Card> SameSetAsCard = item.NeededTokens.Where(x => x.Set.GathererName.ToLower() == item.Set.GathererName.ToLower());
                    Token originalToken = SameSetAsCard.Count() > 0 ? SameSetAsCard.FirstOrDefault() as Token :
                    item.NeededTokens.FirstOrDefault() as Token;
                    Token newToken = new Token(item.ID + "t", originalToken, item.Set, originalToken.ImagePath);

                    //Tell it the set has been altered so it won't be copied inside of MagicAlbulm Folder
                    if (item.Set.GathererName.ToLower() != originalToken.Set.GathererName.ToLower())
                    {
                        newToken.IsAltered = true;
                        newToken.OriginalCard = originalToken;
                    }

                    cardsToProcess.Add(newToken);//Add it the List Of Cards
                }
            }

            return cardsToProcess;
        }

        public static List<Card> AddUnwantedTokens(List<Card> CardsToProcess, List<string> SelectedSets)
        {
            var TokensFromSelectedSets = (from i in TokensOnly
                                          join j in SelectedSets on i.Set.GathererName.ToLower() equals j.ToLower()
                                          select i).ToList();

            var TokensToProcess = CardsToProcess.Where(x => x.IsToken);
            var UnwantedTokens = TokensFromSelectedSets.Where(x => !TokensToProcess.Any(y => x.ID == y.CardNumber)).ToList();

            UnwantedTokens.ForEach(x => 
                {
                    x.ID = x.Name; CardsToProcess.Add(x);
                });

            return CardsToProcess;
        }
    }
}
