﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;

namespace MTGRenamer
{
    class SetsList
    {
        private static List<Set> _Sets;
        public static List<Set> Sets
        {
            get
            {
                if (_Sets == null)
                {
                    LoadFile(FilePath);
                }

                return _Sets;
            }
        }

        private static string filePath = Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), "Sets.txt");
        public static string FilePath
        {
            get
            {
                return filePath;
            }
            set
            {
                filePath = value;
            }
        }

        public static IEnumerable<Set> GetOtherSetsWithSameCard(Card card)
        {
            List<Set> sets = new List<Set>();
            sets.AddRange(from i in Database.Cards
                          where i.Name == card.Name && i != card
                          select i.Set);
            return sets;
        }

        public static Set GetSet(string GathererName)
        {
            GathererName = GathererName.Replace("\"", "");
            return Sets.Where(x => x.GathererName.ToLower() == GathererName.ToLower()).FirstOrDefault();
        }

        private static void LoadFile(string filePath)
        {
            _Sets = new List<Set>();

            try
            {
                using (TextReader read = new StreamReader(filePath))
                {
                    string line;

                    while ((line = read.ReadLine()) != null)
                    {
                        line = line.Replace("\"", "");
                        var parsed = line.Split(',');
                        Sets.Add(GetNewSet(parsed));
                    }
                }
            } catch (Exception)
            {
            }
        }
        private static Set GetNewSet(string[] parsed)
        {
            Set newSet;

            switch (parsed.Count())
            {
                case 6:
                    newSet = new Set(parsed[2], parsed[1], Path.Combine(parsed[0], !string.IsNullOrEmpty(parsed[4]) ? parsed[4] : parsed[2]), parsed[3], parsed[5]);
                    break;
                case 5:
                    newSet = new Set(parsed[2], parsed[1], Path.Combine(parsed[0], parsed[4]), parsed[3]);
                    break;
                case 4:
                    newSet = new Set(parsed[2], parsed[1], Path.Combine(parsed[0], parsed[2]), parsed[3]);
                    break;
                case 3:
                    newSet = new Set(parsed[2], parsed[1], Path.Combine(parsed[0], parsed[2]), parsed[1]);
                    break;
                default:
                    newSet = new Set(parsed[2], parsed[1], Path.Combine(parsed[0], parsed[2]), parsed[1]);
                    break;
            }

            return newSet;
        }
    }
}
