﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;

namespace MTGRenamer
{
    public class Token : Card
    {
        #region Properties
        private string _Power;
        bool AltArtIsLetter = false;
        public string ImagePath { get; set; }
        public string Power
        {
            get
            {
                return _Power;
            }
            set
            {
                _Power = value.Replace("*", "X");
            }
        }

        public string AltArtLetter
        {
            get
            {
                int altArtIndex;
                string altArt = base.AltArt;

                if (int.TryParse(base.AltArt, out altArtIndex))
                {
                    int code = (altArtIndex - 1) + (int)'A';
                    altArt = char.ConvertFromUtf32(code).ToString().ToLower();
                }
                return altArt;
            }
        }

        public override string MagicAlbumName
        {
            get
            {
                if (AltArtIsLetter)
                {
                    _MagicAlbumName = SafeName.Replace(" // ", "_");
                    _MagicAlbumName = _MagicAlbumName.Replace("Ae", "AE");
                    _MagicAlbumName = !string.IsNullOrEmpty(AltArt) ? string.Format("{0} [{1}]", _MagicAlbumName, AltArtLetter) : _MagicAlbumName;
                } else
                {
                    _MagicAlbumName = base.MagicAlbumName;
                }
                return _MagicAlbumName;
            }
        }
        #endregion

        #region Constructor
        public Token(string ID, Token token, Set set, string imagePath)
            : base(token)
        {
            base.ID = ID;
            base.Set = set;
            base.CardNumber = set.GathererName + token.Name + token.AltArt;
            ImagePath = imagePath;
        }

        public Token(string name, string set, string altArt, bool AltArtIsLetter, string imagePath)
            : base(name, set)
        {
            base.ID = set + name + altArt;
            base.CardNumber = set + name + altArt;
            base.AltArt = altArt;
            base.CardText = string.Empty;
            base.Type = string.Format("{0} - {1}", "Token", name);
            ImagePath = imagePath;
            this.AltArtIsLetter = AltArtIsLetter;
        }
        #endregion

        public override bool Download(string savefilePath)
        {
            return base.Download(savefilePath, base.CardNumber, ImagePath);
        }

        public bool DoesCardNeedMe(Card card)
        {
            if (EmblemChecker(card))
                return true;

            bool Needed = card.CardText.ToLower().Contains(string.Format(" {0} ", Name.ToLower()))
            && card.CardText.ToLower().Contains("creature token");

            if (Power == "X/X")
                return Needed;

            return Needed && card.CardText.ToLower().Contains(string.Format(" {0} ", Power).ToLower());
        }

        private bool EmblemChecker(Card card)
        {
            if (card.CardText.ToLower().Contains("emblem") && this.Name.ToLower() == string.Format("emblem {0}", card.Name.ToLower()))
                return true;

            return false;
        }
    }
}
