﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using System.ComponentModel;

namespace MTGRenamer
{
    class DeckList
    {
        #region Fields & Properties
        public List<Deck> Decks
        {
            get
            {
                if (_Decks == null)
                {
                    _Decks = LoadDecks();
                }
                return _Decks;
            }
        }

        private List<Deck> _Decks;
        Deck.Types DeckType;
        string path = string.Empty;
        #endregion

        #region Constructors
        public DeckList(string filepath, Deck.Types type)
        {
            path = filepath;
            DeckType = type;
        }
        #endregion

        #region Load Decks
        private List<Deck> LoadDecks()
        {
            _Decks = new List<Deck>();
            Regex WagicPattern = new Regex(@"deck\d+\." + WagicDeck.defaultExtension, RegexOptions.IgnoreCase);

            switch (DeckType)
            {
                //case Deck.Types.MagicAlbum:
                //    LoadDecks(s => s.EndsWith("." + MagicAlbumDeck.defaultExtension), typeof(MagicAlbumDeck));
                //    break;
                case Deck.Types.Wagic:
                    LoadDecks(s => WagicPattern.Match(s).Success, typeof(WagicDeck));
                    break;
                case Deck.Types.MagicWorkstation:
                    LoadDecks(s => s.EndsWith("." + WorkstationDeck.defaultExtension), typeof(WorkstationDeck));
                    break;
                case Deck.Types.Apprentice:
                    LoadDecks(s => s.EndsWith("." + ApprenticeDeck.defaultExtension), typeof(ApprenticeDeck));
                    break;
                case Deck.Types.MTGO:
                    LoadDecks(s => s.EndsWith("." + MTGODeck.defaultExtension) && !WagicPattern.Match(s).Success, typeof(MTGODeck));
                    break;
                case Deck.Types.Forge:
                    LoadDecks(s => s.EndsWith("." + ForgeDeck.defaultExtension), typeof(ForgeDeck));
                    break;
            }

            SortDecks();
            return _Decks;
        }

        private void LoadDecks(Predicate<string> filter, Type deckClass)
        {
            if (Directory.Exists(path))
            {
                string[] files = Directory.GetFiles(path, "*.*").Where(s => filter(s)).ToArray();
                Array.ForEach(files, x => _Decks.Add((Deck)Activator.CreateInstance(deckClass, x)));
            }
        }
        #endregion

        #region Methods
        private void SortDecks()
        {
            _Decks.Sort((x, y) => new NaturalComparer().Compare(x.FileName, y.FileName));
        }

        internal void Delete(Deck Deck)
        {
            try
            {
                if (Deck != null)
                {
                    if (File.Exists(Deck.FilePath))
                    {
                        File.Delete(Deck.FilePath);
                    }
                    _Decks.Remove(Deck);
                }
            } catch (IOException)
            {
                //File in use 
            }
            catch (UnauthorizedAccessException)
            {
                //No permisssion
            }
        }
        #endregion
    }
}
