﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;

namespace MTGRenamer
{
    internal class ForgeDeck : Deck
    {
        #region Properties
        protected static string _CommentChar = "[";
        protected override string CommentChar
        {
            get
            {
                return _CommentChar;
            }
        }

        public static string defaultExtension = "dck";
        protected override string DefaultExtension
        {
            get
            {
                return defaultExtension;
            }
        }

        public override Deck.Types DeckType
        {
            get
            {
                return Deck.Types.Forge;
            }
        }
        #endregion

        #region Constructors
        public ForgeDeck(string path)
            : base(path)
        {
        }

        public ForgeDeck(Deck deck)
            : base(deck)
        {
        }
        #endregion

        #region Methods
        protected override bool Comments(string line)
        {
            //Ignore Comments
            string NAME = "name=";
            if (line.ToLower().StartsWith("[metadata]") | line.ToLower().StartsWith("[main]") | line.ToLower().StartsWith(NAME))
            {
                string subline = line.Substring(NAME.Length).Trim();
                if (line.ToLower().StartsWith(NAME) && string.IsNullOrEmpty(Name))
                    Name = subline;

                return true;
            }

            return false;
        }

        private bool SideboardReachedLoad = false;
        protected override void LoadFormat(string line, out string name, out string SetID, out string QtyString, out bool SideBoard)
        {
            //4 Tragic Slip|DKA

            //Cards are from the sideboard after a [sideboard] line
            if (line.ToLower().StartsWith("[sideboard]"))
                SideboardReachedLoad = true;

            Regex reg = new Regex(@".*(?<Qty>\d{1,3})(?<name>[^|]*)\|*(?<setID>.*)");
            SideBoard = SideboardReachedLoad;
            name = line.StartsWith(CommentChar) ? string.Empty : reg.Match(line).Groups["name"].Value.Trim();
            SetID = line.StartsWith(CommentChar) ? string.Empty : reg.Match(line).Groups["setID"].Value.Trim();
            QtyString = line.StartsWith(CommentChar) ? string.Empty : reg.Match(line).Groups["Qty"].Value.Trim();
        }

        private bool SideboardReachedWrite = false;
        protected override void SaveFormat(ref StringBuilder sb, DeckCard card)
        {

            //Add an Empty Line before the First Sideboard card.
            if (SideboardReachedWrite == false && card.IsSideboard == true)
            {
                sb.AppendLine("[sideboard]");
                SideboardReachedWrite = true;
            }
            //The Set Information is not written, because if the Set does not exist Forge will crash, we will let him found the cards
            //sb.AppendFormat("{0} {1}|{2}", card.Quantity.ToString(), card.Name, card.Set.WagicID).AppendLine();

            sb.AppendFormat("{0} {1}", card.Quantity.ToString(), card.Name).AppendLine();
        }

        protected override void AddCommentsToDeck(StringBuilder sb)
        {
            sb.AppendLine("[metadata]");
            sb.AppendFormat("Name={0}", Name).AppendLine();
            sb.AppendLine("[main]");
        }

        protected override Card GetCard(string name, string SetID)
        {
            return GetCard(name, SetID, x => x.Set.WagicID.ToUpper() == SetID.ToUpper());
        }
        #endregion
    }
}
