﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;
using System.Reflection;

namespace MTGRenamer
{
    public class MTGODeck : Deck
    {
        #region Properties
        public static string defaultExtension = "txt";
        protected override string DefaultExtension
        {
            get
            {
                return defaultExtension;
            }
        }

        public override Deck.Types DeckType
        {
            get
            {
                return Types.MTGO;
            }
        }
        #endregion

        #region Constructors
        public MTGODeck(Deck source)
            : base(source)
        {
        }
        public MTGODeck(string path)
            : base(path)
        {
        }
        #endregion

        #region Methods
        private bool SideboardReachedLoad = false;
        protected override void LoadFormat(string line, out string name, out string SetID, out string QtyString, out bool SideBoard)
        {
            //4 Path to Exile

            //Cards are from the sideboard after a single empty line
            //Or a Sideboard line
            if (string.IsNullOrEmpty(line) || line.ToLower().StartsWith("sideboard"))
                SideboardReachedLoad = true;

            Regex reg = new Regex(@"^[^\d]*(?<Qty>\d*)(?<name>.*)");
            SideBoard = SideboardReachedLoad;
            name = reg.Match(line).Groups["name"].Value.Trim();
            SetID = string.Empty;
            QtyString = reg.Match(line).Groups["Qty"].Value.Trim();
        }

        private bool SideboardReachedWrite = false;
        protected override void SaveFormat(ref StringBuilder sb, DeckCard card)
        {
            //Add an Empty Line before the First Sideboard card.
            if (SideboardReachedWrite == false && card.IsSideboard == true)
            {
                sb.AppendLine();
                SideboardReachedWrite = true;
            }

            sb.AppendFormat("{0} {1}", card.Quantity.ToString(), card.Name).AppendLine();
        }

        protected override void AddCommentsToDeck(StringBuilder sb)
        {
            //Don't add any comments ovveride the default template
        }
        #endregion
    }
}
