﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;

namespace MTGRenamer
{
    internal class WagicDeck : Deck
    {
        #region Properties
        protected static string _CommentChar = "#";
        protected override string CommentChar
        {
            get
            {
                return _CommentChar;
            }
        }

        public static string defaultExtension = "txt";
        protected override string DefaultExtension
        {
            get
            {
                return defaultExtension;
            }
        }

        public override Deck.Types DeckType
        {
            get
            {
                return Deck.Types.Wagic;
            }
        }
        #endregion

        #region Constructors
        public WagicDeck(string path)
            : base(path)
        {
        }

        public WagicDeck(Deck deck)
            : base(deck)
        {
        }
        #endregion

        #region Methods

        protected override void LoadFormat(string line, out string name, out string SetID, out string QtyString, out bool SideBoard)
        {
            // Card is a name
            Regex reg = new Regex(@"(?<name>^[^(\d\*]+)(?:.*(?<setID>(?<=\()[^)]+))?");
            SideBoard = false;
            name = reg.Match(line).Groups["name"].Value.Trim();
            SetID = reg.Match(line).Groups["setID"].Value.Trim();
            QtyString = Regex.Match(line, @".*(?<Qty>(?<=\*\s*)\d+)").Groups["Qty"].Value.Trim();
        }

        protected override void SaveFormat(ref StringBuilder sb, DeckCard card)
        {
            //Full Name Format (Put as a comment)
            sb.AppendFormat("{0}{1} ({2}) *{3}", CommentChar, card.Name, card.Set.WagicID, card.Quantity.ToString()).AppendLine();

            //ID Format
            string cardID = card.IsSideboard ? String.Format("{0}{1}", CommentChar, card.ID) : card.ID;
            for (int i = 0; i < card.Quantity; i++)
                sb.AppendLine(cardID);

            //Empty Line
            sb.AppendLine();
        }

        protected override Card GetCard(string name, string SetID)
        {
            return GetCard(name, SetID, x => x.Set.WagicID.ToUpper() == SetID.ToUpper());
        }
        #endregion
    }
}
