﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;

namespace MTGRenamer
{
    internal class WorkstationDeck : Deck
    {
        #region Properties
        public static string defaultExtension = "mwDeck";
        protected override string DefaultExtension
        {
            get
            {
                return defaultExtension;
            }
        }

        public override Deck.Types DeckType
        {
            get
            {
                return Deck.Types.MagicWorkstation;
            }
        }
        #endregion

        #region Constructors
        public WorkstationDeck(string path)
            : base(path)
        {
        }

        public WorkstationDeck(Deck deck)
            : base(deck)
        {
        }
        #endregion

        #region Methods
        protected override void LoadFormat(string line, out string name, out string SetID, out string QtyString, out bool SideBoard)
        {
            //    4 [CFX] Path to Exile
            Regex reg = new Regex(@"^\s*(?<Qty>\d*)(?:.+\[(?<setID>.+)\])*(?<name>[^(]*)");
            SideBoard = line.Trim().ToUpper().StartsWith("SB:");
            name = reg.Match(line).Groups["name"].Value.Trim();
            SetID = reg.Match(line).Groups["setID"].Value.Trim();
            QtyString = reg.Match(line).Groups["Qty"].Value.Trim();
        }

        protected override void SaveFormat(ref StringBuilder sb, DeckCard card)
        {
            //Full Name Format
            string Prefix = card.IsSideboard ? "SB: " : "    ";
            sb.AppendFormat("{0}{1} [{2}] {3}", Prefix, card.Quantity.ToString(), card.Set.WorkstationID, card.Name).AppendLine();
        }

        protected override void AddCommentsToDeck(StringBuilder sb)
        {
            sb.AppendFormat("{0} Deck file for Magic Workstation (http://www.magicworkstation.com)", CommentChar).AppendLine();
            base.AddCommentsToDeck(sb);
        }
        #endregion
    }
}
