﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;
using System.IO;
using CheckComboBox;

namespace MTGRenamer
{
    public partial class frmMain : Form
    {
        #region Fields
        Thread RenameDownload;
        List<Card> CardsToProcess;
        bool ManualCheck = true;
        #endregion

        #region Constructors
        public frmMain()
        {
            InitializeComponent();
            this.Text = Utils.GetTitle();
            this.Icon = Utils.GetIcon();
            LoadSettings();
        }
        #endregion

        #region Events
        private void btnOpenDB_Click(object sender, EventArgs e)
        {
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                LoadDatabase(openFile.FileName);
            }
        }

        private void btnGO_Click(object sender, EventArgs e)
        {
            if (Database.Cards == null || Database.Cards.Count == 0)
            {
                MessageBox.Show("Load Database Before");
            } else
            {
                Renamer rn = new Renamer(txtSource.Text, txtDestination.Text);
                rn.Progress += new EventHandler<ProgressEventArgs>(rn_Progress);

                GetCardsToProcess();

                if (CardsToProcess != null && CardsToProcess.Count() > 0)
                {
                    Utils.LocationType source = (Utils.LocationType)cbSource.SelectedItem;
                    Utils.LocationType dest = (Utils.LocationType)cbDestination.SelectedItem;

                    if (cbAddWagicRealName.Checked)
                        if (dest == Utils.LocationType.Wagic)
                        {
                            AddWagicRealName(CardsToProcess, txtDestination.Text);
                            return;
                        } else
                        {
                            MessageBox.Show("Make sure that Wagic is Set as the Destination Type");
                            return;
                        }

                    RenameDownload = new Thread(() => rn.DoWork(CardsToProcess, source, dest));
                    RenameDownload.Start();

                    SaveSettings();
                } else
                {
                    MessageBox.Show("No Sets selected to Process");
                }
            }
        }

        void rn_Progress(object sender, ProgressEventArgs e)
        {
            UpdateProgress(e.CurrentFile, e.Maximum, e.Message);
        }

        private void btnSourcePath_Click(object sender, EventArgs e)
        {
            if (folderBrowser.ShowDialog() == DialogResult.OK)
            {
                txtSource.Text = folderBrowser.SelectedPath;
            }
        }

        private void btnDestination_Click(object sender, EventArgs e)
        {
            if (folderBrowser.ShowDialog() == DialogResult.OK)
            {
                txtDestination.Text = folderBrowser.SelectedPath;
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (RenameDownload != null)
                RenameDownload.Abort();

            progress.Value = 0;
            UpdateProgress("Aborting Job, Cleaning up");
        }
        private void frmMain_Load(object sender, EventArgs e)
        {
            LoadDatabase();
            cbSource.DataSource = (Enum.GetValues(typeof(Utils.LocationType)));
            cbDestination.DataSource = (Enum.GetValues(typeof(Utils.LocationType)));
            cbSource.SelectedIndex = Properties.Settings.Default.InputType;
            cbDestination.SelectedIndex = Properties.Settings.Default.OutputType;
        }

        private void btnInvert_Click(object sender, EventArgs e)
        {
            string source = txtSource.Text;
            string destination = txtDestination.Text;

            Utils.LocationType sourceType = (Utils.LocationType)cbSource.SelectedItem;
            Utils.LocationType destType = (Utils.LocationType)cbDestination.SelectedItem;

            //Invert all data
            txtSource.Text = destination;
            txtDestination.Text = source;
            cbDestination.SelectedItem = sourceType;
            cbSource.SelectedItem = destType;
        }

        private void convertDeckToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (radSource.Checked == true)
            {
                LoadDeckEditor(txtSource.Text, (Utils.LocationType)cbSource.SelectedItem);
            } else
            {
                LoadDeckEditor(txtDestination.Text, (Utils.LocationType)cbDestination.SelectedItem);
            }
        }

        private void cbSets_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            CheckedListBox ccb = (CheckedListBox)sender;

            if (ManualCheck)
            {
                if (e.Index == 0)
                {
                    bool newValue = e.NewValue == CheckState.Checked ? true : false;
                    ManualCheck = false;
                    for (int i = 1; i < ccb.Items.Count; i++)
                    {
                        cbSets.SetItemChecked(i, newValue);
                    }
                    ManualCheck = true;
                } else
                {
                    ManualCheck = false;
                    cbSets.SetItemChecked(0, false);
                    ManualCheck = true;
                }
            }
        }


        #endregion

        #region Methods
        private List<Card> GetCardsToProcess()
        {
            CardsToProcess = new List<Card>();

            if (!cbAddWagicRealName.Checked)
            {
                UpdateProgress("Grabbing Tokens List From Magiccards.info");
                Application.DoEvents();
                Database.LoadTokensOnly();
            }

            List<string> SelectedSets = new List<string>();
            foreach (string item in cbSets.CheckedItems)
                SelectedSets.Add(item);

            CardsToProcess = (from i in Database.Cards
                              join j in SelectedSets on i.Set.GathererName equals j
                              select i).ToList();

            UpdateProgress("Processing Tokens");
            CardsToProcess = Database.AddTokensReverse(CardsToProcess);
            CardsToProcess = Database.AddUnwantedTokens(CardsToProcess, SelectedSets);

            return CardsToProcess;
        }

        private void AddWagicRealName(IEnumerable<Card> setsToProcess, string path)
        {
            WagicSetNamer namer = new WagicSetNamer(path);
            IEnumerable<Set> sets = from i in setsToProcess
                                    group i by i.Set.GathererName
                                        into g
                                        select new Set(g.Key);

            Thread RealName = new Thread(() => namer.Rename(sets));
            RealName.Start();
        }

        private void LoadDeckEditor(string ImagePath, Utils.LocationType type)
        {
            CardImage.ImagePath = ImagePath;
            CardImage.ImageType = type;
            SaveSettings();

            frmDeckEditor frm = new frmDeckEditor();
            frm.Show();
        }
        #endregion

        #region Settings
        private void LoadSettings()
        {
            txtSource.Text = Properties.Settings.Default.InputDir;
            txtDestination.Text = Properties.Settings.Default.OutputDir;
        }

        private void SaveSettings()
        {
            //Save the settings only when it is Default
            Properties.Settings.Default.InputDir =
                //!radSource.Checked ? string.Empty ://this line is used if you want the settings saved only when Default
            string.IsNullOrEmpty(txtSource.Text) ? string.Empty : txtSource.Text;
            Properties.Settings.Default.OutputDir =
                //!radDestination.Checked ? string.Empty ://this line is used if you want the settings saved only when Default
            string.IsNullOrEmpty(txtDestination.Text) ? string.Empty : txtDestination.Text;
            Properties.Settings.Default.OutputType = cbDestination.SelectedIndex;
            Properties.Settings.Default.InputType = cbSource.SelectedIndex;
            Properties.Settings.Default.Save();
        }
        #endregion

        #region Progress Bar
        private void UpdateProgress(string Message)
        {
            UpdateProgress(0, 0, Message);
        }

        private delegate void UpdateProgressDelegate(int Value, int Maximum, string Message);
        private void UpdateProgress(int Value, int Maximum, string Message)
        {
            // Lock this block of code to one thread at a time because of multiple
            // threads accessing it and updating control.
            lock (this)
            {
                // Check to see if the thread that is trying to access this code
                // is the GUI thread or another thread.
                if (progress.InvokeRequired)
                {
                    // The thread that entered this code in not the thread that create
                    // the control. Create a deleagte and then execute this procedure
                    // from the GUI thread.
                    UpdateProgressDelegate del = new UpdateProgressDelegate(UpdateProgress);
                    progress.BeginInvoke(del, Value, Maximum, Message);
                } else
                {
                    // The thread in this section of code is the GUI thread
                    progress.Value = Maximum == 0 ? progress.Value : Value;
                    progress.Maximum = Maximum == 0 ? progress.Maximum : Maximum;
                    lblProgressMessage.Text = Message + "\n";
                    Application.DoEvents();
                }
            }
        }
        #endregion

        #region Load Database
        private void LoadDatabase()
        {
            string path = String.Empty;
            LoadDatabase(path);
        }

        private void LoadDatabase(string path)
        {
            if (!string.IsNullOrEmpty(path))
                Database.LoadDatabase(path);

            cbSets.Items.Clear();
            cbSets.Items.Add("ALL");
            cbSets.Items.AddRange(Database.Cards.GroupBy(x => x.Set.GathererName).Select(x => x.Key).ToArray());
        }
        #endregion
    }
}
