﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace MTGRenamer
{
    internal class WagicSetNamer
    {
        Dictionary<string, string> DatFiles;

        public WagicSetNamer(string path)
        {
            DatFiles = new Dictionary<string, string>();
            foreach (var item in Directory.GetFiles(path, "_cards.dat", SearchOption.AllDirectories))
            {
                string Set;
                try
                {
                    Set = Regex.Match(item, @"[^\\]*(?=\\[^\\]*$)").Value;
                    DatFiles.Add(Set, item);
                } catch (Exception)
                {
                }
            }
        }

        public void Rename(IEnumerable<Set> Sets)
        {
            var join = from i in Sets
                       join j in DatFiles on i.WagicID equals j.Key
                       select new
                       { Set = i, DatFile = j };

            if (join.Count() > 0)
            {
                foreach (var item in join)
                {
                    EditFile(item.DatFile.Value, item.Set);
                }

                System.Windows.Forms.MessageBox.Show("Renaming is Done");
            } else
            {
                System.Windows.Forms.MessageBox.Show("No _cards.dat files found in Destination path");
            }
        }

        private void EditFile(string path, Set SetName)
        {
            if (SetName != null && File.Exists(path))
                WriteFile(path, ReadFile(path, SetName));
        }

        private string ReadFile(string path, Set set)
        {
            try
            {
                StringBuilder datFile = new StringBuilder();
                foreach (string line in File.ReadAllLines(path))
                {
                    if (line.StartsWith("name"))
                        continue;

                    if (line.StartsWith("[/meta]"))
                    {
                        datFile.AppendLine(string.Format("name={0}", set.GathererName));
                        datFile.AppendLine("[/meta]");
                        continue;
                    }

                    datFile.AppendLine(line);
                }

                return datFile.ToString();
            } catch (Exception)
            {
                return null;
            }
        }

        private void WriteFile(string path, string Value)
        {
            try
            {
                using (TextWriter writer = new StreamWriter(path))
                {
                    writer.Write(Value);
                }
            } catch (Exception)
            {
            }
        }
    }
}
